"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cluster = require("cluster");
const commander_1 = require("commander");
// Touch all the files from the main index, otherwise workers can't access all the necessary file
require("..");
const configFileReader_1 = require("../config/configFileReader");
const configInterpreter_1 = require("../config/configInterpreter");
const debugTools_1 = require("../helpers/debugTools");
const fileSystemHelpers_1 = require("../helpers/fileSystemHelpers");
const typeCheckHelpers_1 = require("../helpers/typeCheckHelpers");
const program = new commander_1.Command();
program
    .usage("[options] [projects/tsconfigs...]")
    .usage("[options] [projects/tsconfigs...]")
    .option("-d, --debug", "Add way too much logging")
    .option("-c, --config [path_to_config]", "Path to mtsc config")
    .option("-w, --watch", "Watch the given projects (default false)")
    .option("-p, --preserveWatchOutput", "Don't throw away watch output (default true in debug mode)")
    .option("-t, --tsc [path_to_tsc]", "Path to compiler for all projects (will search in exec dir if not given)")
    .option("--noEmit", "Do not emit outputs")
    .parse(process.argv);
const options = program.opts();
(0, debugTools_1.setDebugMode)(!!options.debug);
function initProjectWatcherFromCli() {
    const foundConfig = (0, configFileReader_1.findMtscConfig)(options.config);
    const mtscConfig = foundConfig || { projects: [] };
    mtscConfig.debug = options.debug || mtscConfig.debug;
    (0, debugTools_1.setDebugMode)(!!mtscConfig.debug);
    if (foundConfig)
        (0, configFileReader_1.validateMtscConfig)(foundConfig);
    (0, debugTools_1.debugLog)("Checking if config is given or autodetect", options.config);
    (0, debugTools_1.debugLog)("Any options given in CLI will overwrite the config");
    (0, debugTools_1.debugLog)("Checking if global compiler is given", options.tsc);
    if (options.tsc && (0, typeCheckHelpers_1.isValidString)(options.tsc)) {
        (0, debugTools_1.debugLog)("Global compiler set to", options.tsc);
        mtscConfig.compiler = options.tsc;
    }
    else if (options.tsc && (0, typeCheckHelpers_1.isValidBoolean)(options.tsc)) {
        (0, debugTools_1.debugLog)("Compiler command is true, so now searching for tsc executable");
        mtscConfig.compiler = (0, fileSystemHelpers_1.findNodeModuleExecutable)(".", "tsc");
        (0, debugTools_1.debugLog)("Compiler command set to", mtscConfig.compiler);
    }
    else if (options.tsc) {
        (0, debugTools_1.debugLog)("Invalid tsc option given", options.tsc);
        throw new Error("Invalid tsc option given");
    }
    if (options.watch) {
        (0, debugTools_1.debugLog)("Global watch set to", options.watch);
        mtscConfig.watch = options.watch;
    }
    if (options.noEmit) {
        (0, debugTools_1.debugLog)("Global noEmit set to", options.noEmit);
        mtscConfig.noEmit = options.noEmit;
    }
    (0, debugTools_1.debugLog)("Checking if there are project folders or tsconfigs given", options.args);
    if (!options.args.length && !mtscConfig.projects.length && !mtscConfig.useYarnWorkspaces) {
        (0, debugTools_1.debugLog)("No tsconfig arguments given, will use current dir");
        options.args.push(".");
    }
    program.args.forEach(path => {
        mtscConfig.projects.push({ path });
    });
    return (0, configInterpreter_1.initProjectsWatcher)(mtscConfig);
}
if (cluster.isMaster) {
    initProjectWatcherFromCli().startCompilations();
}
