"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initProjectsWatcher = void 0;
const debugTools_1 = require("../helpers/debugTools");
const fileSystemHelpers_1 = require("../helpers/fileSystemHelpers");
const typeCheckHelpers_1 = require("../helpers/typeCheckHelpers");
const ProjectsWatcher_1 = require("../projectWatcher/ProjectsWatcher");
const yarnHelpers_1 = require("../helpers/yarnHelpers");
function initProjectsWatcher(mtscCfg) {
    (0, debugTools_1.setDebugMode)(!!mtscCfg.debug);
    if (!(0, typeCheckHelpers_1.isValidBoolean)(mtscCfg.watch))
        mtscCfg.watch = false;
    const projectsWatcher = new ProjectsWatcher_1.ProjectsWatcher();
    if (mtscCfg.useYarnWorkspaces)
        (0, yarnHelpers_1.getYarnWorkspaces)().forEach(workspace => mtscCfg.projects.push(workspace));
    const normalisedProjects = mtscCfg.projects.map(project => ((0, typeCheckHelpers_1.isValidString)(project) ? { path: project } : project));
    const uniqueProjects = normalisedProjects.reduce((projects, project) => (projects.find(p => p.path === project.path) ? projects : [...projects, project]), new Array());
    for (const projectCfg of uniqueProjects) {
        if (!(0, typeCheckHelpers_1.isValidString)(projectCfg.compiler)) {
            projectCfg.compiler = (0, typeCheckHelpers_1.isValidString)(mtscCfg.compiler) ? mtscCfg.compiler : (0, fileSystemHelpers_1.findNodeModuleExecutable)(projectCfg.path, "tsc");
        }
        if (!(0, typeCheckHelpers_1.isValidBoolean)(projectCfg.noEmit))
            projectCfg.noEmit = mtscCfg.noEmit;
        (0, debugTools_1.debugLog)(`Adding project:\nPath: ${projectCfg.path}\nwatch: ${!!mtscCfg.watch}\nCompiler: ${projectCfg.compiler}`);
        const projectSettings = {
            watch: mtscCfg.watch,
            path: projectCfg.path,
            compiler: projectCfg.compiler,
            noEmit: projectCfg.noEmit
        };
        projectsWatcher.addWorker(projectSettings);
    }
    return projectsWatcher;
}
exports.initProjectsWatcher = initProjectsWatcher;
