"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Project = exports.TSC_ERRORS_FOUND = void 0;
const sh = require("shelljs");
const debugTools_1 = require("../helpers/debugTools");
const SH_EXECOPTIONS = { async: true, silent: true };
const TSC_COMPILATION_COMPLETE = /(?:Compilation complete\.|Found \d+ errors?\.) Watching for file changes/;
const TSC_COMPILATION_STARTED = /File change detected. Starting incremental compilation|Starting compilation in watch mode.../;
exports.TSC_ERRORS_FOUND = /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error)\s+(TS\d+)\s*:\s*(.*)$/gm;
class Project {
    constructor(args, reportState) {
        this.resultBuffer = null;
        this.lastResult = "";
        this.processInstruction = (instruction) => {
            if (instruction === "START")
                this.startCompiling();
        };
        this.parseCommandOutput = (data) => {
            if (data.match(TSC_COMPILATION_COMPLETE)) {
                this.processCompilationComplete();
                return;
            }
            // No result buffer? Then don't forget to report that we have started
            if (!this.resultBuffer) {
                this.lastResult = "";
                this.resultBuffer = [];
                this.sendStateUpdate("COMPILING");
            }
            if (data) {
                // Ignore the start compiling message, the ProjectsWatcher will write this only if needed.
                if (data.match(TSC_COMPILATION_STARTED))
                    return;
                this.resultBuffer.push(data);
            }
        };
        this.processCompilationComplete = () => {
            // Can't proces a complete compilation if there was no message
            if (!this.resultBuffer)
                return;
            this.flushResultBuffer();
            this.sendStateUpdate("COMPLETE");
        };
        this.projectSettings = args;
        this.reportState = reportState;
    }
    sendStateUpdate(state) {
        this.reportState({ lastResult: this.lastResult, projectState: state });
    }
    startCompiling() {
        var _a, _b;
        const compileCommand = createCompileCommand(this.projectSettings);
        (0, debugTools_1.debugLog)("Tsc: executing following command", compileCommand);
        const child = sh.exec(compileCommand, SH_EXECOPTIONS);
        (_a = child.stdout) === null || _a === void 0 ? void 0 : _a.on("data", this.parseCommandOutput);
        (_b = child.stdout) === null || _b === void 0 ? void 0 : _b.on("end", this.processCompilationComplete);
    }
    flushResultBuffer() {
        this.lastResult = this.resultBuffer.join("");
        this.resultBuffer = null;
    }
}
exports.Project = Project;
function createCompileCommand(projectSettings) {
    const { compiler, watch, path, noEmit } = projectSettings;
    const commandAndFlags = [
        compiler,
        watch ? "-w" : "",
        "--pretty false",
        "--preserveWatchOutput",
        noEmit === true ? "--noEmit" : "",
        `-p ${path}`
    ];
    return commandAndFlags.filter(value => !!value).join(" ");
}
